/*******************************************************************

   Program:			CDemo
	
	Description:   Main program for the PhotoV C Demo application.

	Author:			Wade H. Cobb

	Date:				2/20/93

********************************************************************/	
#define STRICT
#include <windows.h>
#include <commdlg.h>
#include <dos.h>
#include <stdlib.h>
#include <direct.h>
#include <string.h>
#include "resdefs.h"
#include "internal.h"
#include "photov.h"

   // Module common vars
static nEffect;            // Current checked effect menu item

   // Module functions
static void doOpenFile(HWND hWnd);
static void doMenus(HWND hWnd,LPARAM lParam);

   // Macros
#define ENABLE(i) EnableMenuItem(GetMenu(hWnd),i,MF_BYCOMMAND|MF_ENABLED)
#define DISABLE(i) EnableMenuItem(GetMenu(hWnd),i,MF_BYCOMMAND|MF_GRAYED)
#define CHECKEFFECT(i) CheckMenuItem(GetSubMenu(GetMenu(hWnd),3),i,MF_BYPOSITION|MF_CHECKED)
#define UNCHECK(i) CheckMenuItem(GetMenu(hWnd),i,MF_BYCOMMAND|MF_UNCHECKED)
#define UNCHECKEFFECT(i) CheckMenuItem(GetSubMenu(GetMenu(hWnd),3),i,MF_BYPOSITION|MF_UNCHECKED)
#define CHECK(i) CheckMenuItem(GetMenu(hWnd),i,MF_BYCOMMAND|MF_CHECKED)



/********************************************************************

                       Application entry point

*********************************************************************/
int PASCAL WinMain( HINSTANCE hInstance,
                    HINSTANCE hPrevInstance,
                    LPSTR lpszCmdLine,
                    int nCmdShow)
{
   MSG      msg;
   HANDLE   hAccel;

         // Save application instance in global variable
      hInstMain=hInstance;


   if (!hPrevInstance)
	{
         // Register all of our windows
      if (regWindows())
         return 0;         // Registration failed - exit 

   }

      // Load accelerator keys defined in resource file
   hAccel=LoadAccelerators(hInstMain,"MainAccelTable");

      // Create the main window
   hWndFrame = CreateWindow(szAppName, "CDemo",
                        WS_OVERLAPPEDWINDOW,
                        CW_USEDEFAULT, CW_USEDEFAULT,
                        CW_USEDEFAULT, CW_USEDEFAULT,
                        NULL, NULL, hInstance, lpszCmdLine);

   if (hWndFrame)
   {
         // Display the main window                           
      ShowWindow(hWndFrame, nCmdShow);

         // Force an update of the main window
      UpdateWindow(hWndFrame); 
   }
   else
      return 0;

      // Force the cphotov dll to load
   registerPhotoV();

      // Initialize default directory name
   lstrcpy(szTheDirName,"");

      // Initialize file open flag
   bTheFileOpen=FALSE;

      // Process Windows messages until application terminates
   while (GetMessage(&msg, NULL, 0, 0))
   {
      if (!TranslateAccelerator(hWndFrame,hAccel,&msg))
      {
         TranslateMessage(&msg);
         DispatchMessage(&msg);
      }
   }

   return msg.wParam;
}


/***********************************************************************

         Main window frame Procedure

************************************************************************/
long WINAPI WndProc(WNDPARMS)
{
   long           lRet=0L;

   switch (message)
   {
      case WM_CREATE:

            // Create a photov window
         hWndPhotoV=CreateWindow("PhotoV","PhotoV Window",
                                 WS_CHILD|WS_VISIBLE|WS_VSCROLL|WS_HSCROLL,
                                 0,0,0,0,
                                 hWnd,
                                 (HMENU)1,
                                 hInstMain,NULL);
         if (hWndPhotoV==NULL)
         {
            return -1;
         }

            // Check the (none) effect item
         nEffect=0;
         CHECKEFFECT(nEffect);

            // Check gray for background color
         SendMessage(hWndPhotoV,ICM_SETBACKCOLOR,0,(LPARAM)RGB(192,192,192));
         CHECK(IDM_GRAY);
         return 0;

      case WM_COMMAND:
         switch (wParam)
         {
            case IDM_ABOUT:
               DialogBox(hInstMain,"ABOUT",hWnd,(DLGPROC)AboutDlgProc);
               break;

            case IDM_EXIT:
               DestroyWindow(hWnd);
               break;

            case IDM_OPEN:
               doOpenFile(hWnd);
               break;

            case IDM_PRINT:
               SendMessage(hWndPhotoV,ICM_SETCOMMAND,COMMANDPRINT,0L);
               break;

            case IDM_COPY:
               SendMessage(hWndPhotoV,ICM_SETCOMMAND,COMMANDCOPY,0L);
               break;

            case IDM_BESTFIT:
               SendMessage(hWndPhotoV,ICM_SETSCALE,SCALEBESTFIT,0L);
               break;

            case IDM_FITVERT:
               SendMessage(hWndPhotoV,ICM_SETSCALE,SCALEFITVERT,0L);
               break;

            case IDM_FITHORZ:
               SendMessage(hWndPhotoV,ICM_SETSCALE,SCALEFITHORZ,0L);
               break;

            case IDM_FIT1TO1:
               SendMessage(hWndPhotoV,ICM_SETSCALE,SCALE1TO1,0L);
               break;

            case IDM_ZOOMIN:
               SendMessage(hWndPhotoV,ICM_SETCOMMAND,COMMANDZOOMIN,0L);
               break;

            case IDM_ZOOMOUT:
               SendMessage(hWndPhotoV,ICM_SETCOMMAND,COMMANDZOOMOUT,0L);
               break;

            case IDM_INCREASE:
               SendMessage(hWndPhotoV,ICM_SETCOMMAND,COMMANDINCREASE,0L);
               break;

            case IDM_DECREASE:
               SendMessage(hWndPhotoV,ICM_SETCOMMAND,COMMANDDECREASE,0L);
               break;

            case IDM_ROTATE0:
               SendMessage(hWndPhotoV,ICM_SETROTATION,ROTATE0,0L);
               break;

            case IDM_ROTATE90:
               SendMessage(hWndPhotoV,ICM_SETROTATION,ROTATE90,0L);
               break;

            case IDM_ROTATE180:
               SendMessage(hWndPhotoV,ICM_SETROTATION,ROTATE180,0L);
               break;

            case IDM_ROTATE270:
               SendMessage(hWndPhotoV,ICM_SETROTATION,ROTATE270,0L);
               break;

            case IDM_BLACK:
               UNCHECK(IDM_GRAY);
               UNCHECK(IDM_WHITE);
               CHECK(IDM_BLACK);
               SendMessage(hWndPhotoV,ICM_SETBACKCOLOR,0,(LPARAM)RGB(0,0,0));
               break;

            case IDM_WHITE:
               UNCHECK(IDM_GRAY);
               CHECK(IDM_WHITE);
               UNCHECK(IDM_BLACK);
               SendMessage(hWndPhotoV,ICM_SETBACKCOLOR,0,(LPARAM)RGB(255,255,255));
               break;
 
            case IDM_GRAY:
               CHECK(IDM_GRAY);
               UNCHECK(IDM_WHITE);
               UNCHECK(IDM_BLACK);
               SendMessage(hWndPhotoV,ICM_SETBACKCOLOR,0,(LPARAM)RGB(192,192,192));
               break;
         }
         if (wParam>=IDM_FX0 && wParam<=IDM_FX16)
         {
               // Uncheck the current effect item
            UNCHECKEFFECT(nEffect);
            nEffect=wParam-IDM_FX0;
            CHECKEFFECT(nEffect);

               // Tell the photov control
            if (hWndPhotoV)
               SendMessage(hWndPhotoV,ICM_SETEFFECT,0,(LPARAM)nEffect);
         }
         break;

      case WM_INITMENUPOPUP:
         doMenus(hWnd,lParam);
         break;

      case WM_SIZE:

            // Make photov window same size as client
         MoveWindow(hWndPhotoV,0,0,LOWORD(lParam),HIWORD(lParam),TRUE);
         break;

      case WM_CLOSE:
         DestroyWindow(hWnd);
         break;

      case WM_DESTROY:
         PostQuitMessage(0);
         break;

      default:
         lRet=DefWindowProc(hWnd, message, wParam, lParam);
         break;
   }
   return lRet;
}

/****************************************************************

        AboutDlgProc

*****************************************************************/
long WINAPI AboutDlgProc(WNDPARMS)
{
   switch (message)
   {
      case WM_INITDIALOG:
         break;

      case WM_COMMAND:
         switch (wParam)
         {
            case IDOK:
               EndDialog(hWnd,TRUE);
               break;

            case IDCANCEL:
               EndDialog(hWnd,FALSE);
               break;

            default:
               return FALSE;
         }
      default:
         return FALSE;
   }
   return TRUE;
}

/********************************************************************

      doOpenFile

      Ask the user for a file name and open the image file.

*********************************************************************/
static void doOpenFile(HWND hWnd)
{
   OPENFILENAME      ofn;
   char              szFile[_MAX_PATH+1];
static   char        szFileTitle[_MAX_PATH+1];
   char              szFilter[256];
   int               n;
static int           nFilterIndex=1;


      // Open the file dialog box

   if (lstrlen(szTheDirName)==0)
   {
         // Initialize directory to current default
      _getdcwd(0,szTheDirName,_MAX_PATH);
   }

   LoadString(hInstMain,IDS_FILTER,szFilter,sizeof(szFilter));

   for (n=0;szFilter[n]!='\0';n++)
   {
      if (szFilter[n]=='|')
         szFilter[n]='\0';
   }

   _fmemset(&ofn,0,sizeof(OPENFILENAME));
   lstrcpy(szFile,szTheDirName);

   szFile[0]='\0';
   ofn.lStructSize=sizeof(OPENFILENAME);
   ofn.hwndOwner=hWnd;
   ofn.lpstrFilter=szFilter;
   ofn.lpstrFile=szFile;
   ofn.nMaxFile=sizeof(szFile);
   ofn.lpstrFileTitle=szFileTitle;
   ofn.nMaxFileTitle=sizeof(szFileTitle);
   ofn.lpstrInitialDir=szTheDirName;
   ofn.Flags=OFN_PATHMUSTEXIST|OFN_FILEMUSTEXIST;
   ofn.nFilterIndex=nFilterIndex;

   if (GetOpenFileName(&ofn))
   {
         // Save the directory
      lstrcpy(szTheDirName,ofn.lpstrFile);
      n=lstrlen(szTheDirName)-1;
      while ( (szTheDirName[n]!='\\') && n--);
      if (n>0)
         szTheDirName[n]='\0';

      nFilterIndex=(int)ofn.nFilterIndex;

         // lpstrFile contains the file name
      SendMessage(hWndPhotoV,ICM_SETFILE,0,(LPARAM)(LPSTR)ofn.lpstrFile);

         // Let everyone know we have a file open
      bTheFileOpen=TRUE;
   }
}

/********************************************************************

      doMenus

      Set the menu states for all the menus

*********************************************************************/
static void doMenus(HWND hWnd,LPARAM lParam)
{
   char  szMenus[MAXMENUS+1];
   int   ix;
typedef struct {
   int   nMenuID;
   int   nMenuChar;
   } VIEWMENUS;

   VIEWMENUS men[]={{IDM_BESTFIT,MENUBESTFIT},
                {IDM_FITVERT,MENUFITVERT},
                {IDM_FITHORZ,MENUFITHORZ},
                {IDM_FIT1TO1,MENU1TO1},
                {IDM_INCREASE,MENUINCREASECONTRAST},
                {IDM_DECREASE,MENUDECREASECONTRAST},
                {IDM_ZOOMIN,MENUZOOMIN},
                {IDM_ZOOMOUT,MENUZOOMOUT},
                {IDM_ROTATE0,MENUROTATE0},
                {IDM_ROTATE90,MENUROTATE90},
                {IDM_ROTATE180,MENUROTATE180},
                {IDM_ROTATE270,MENUROTATE270}};

      // Get the menu settings from PhotoV

      if (hWndPhotoV)
      {
         SendMessage(hWndPhotoV,ICM_GETPROPERTY,IVAL_MENUS,(LPARAM)(LPSTR)szMenus);
      }
      else
      return;

   switch (LOWORD(lParam))
   {
      case 0:        // File menu
         if (bTheFileOpen)
            ENABLE(IDM_PRINT);
         else
            DISABLE(IDM_PRINT);
         break;

      case 1:        // Edit menu
         if (bTheFileOpen && szMenus[MENUCOPY]=='1')
            ENABLE(IDM_COPY);
         else
            DISABLE(IDM_COPY);
         break;

      case 2:        // View menu
         for (ix=0;ix<(sizeof(men)/sizeof(VIEWMENUS));ix++)
         {
            if (!bTheFileOpen || szMenus[men[ix].nMenuChar]=='0')
            {
               DISABLE(men[ix].nMenuID);
            }
            else
               ENABLE(men[ix].nMenuID);
         }
         break;

      case 3:        // Effects menu - (always enabled)
         break;
   }
}
